C THIS IS A GROUP OF SUBROUTINES DESIGNED TO FACILITATE HANDLING
C INTERACTIVE COMMANDS AND DATA INPUT FROM A TERMINAL.
C
C FOR INFORMATION SEE DOC:TIOPCKG.DOC
C
C
C WRITTEN BY 
C	J. KRISTIAN
C	MT. WILSON OBSERVATORY
C	813 SANTA BARBARA STREET
C	PASADENA, CALIF. 91101
C

	SUBROUTINE GETCMD(PROMPT,CMDLIS,CMD,NWD)
C
C  ON 12/20/81, CONVERTED THIS ROUTINE TO GETCMD2 BY ADDING
C  CAPITALIZATION OPTION, AND MADE GETCMD A SIMPLE PASS-THROUGH
C  ROUTINE.
C
	CHARACTER PROMPT*(*), CMDLIS*(*), CMD*(*)
	LOGICAL CAPSONLY
C
	CAPSONLY = .FALSE.
	CALL GETCMD2(PROMPT,CMDLIS,CAPSONLY,CMD,NWD)
C
	RETURN
	END

	SUBROUTINE GETCMD2(PROMPT,CMDLIS,CAPSONLY,CMD,NWD)
C
C THIS IS GETCMD, GENERALIZED 12/20/81 BY ADDING CAPSONLY OPTION.
C
C USED TO FORCE A RESPONSE (A COMMAND), FROM THE USER AT A TERMINAL,
C WHICH MATCHES A COMMAND IN A GIVEN LIST (CMDLIS).
C THE FIRST TIME THROUGH, ASKS FOR A WORD USING THE
C STRING "PROMPT".  IF A MATCH IS FOUND TO THE COMMAND LIST, RETURNS THE
C NUMBER OF THE COMMAND LIST WORD (NWD) AND THE FULL WORD ITSELF (CMD).
C OTHERWISE, CONTINUES TO PROMPT FOR
C A NEW INPUT WORD USING APPROPRIATE ERROR MESSAGES.
C
C INPUT PARAMETERS: CHARACTER STRINGS "PROMPT" AND "CMDLIS".
C			LOGICAL CAPSONLY.
C
C TERMINAL INPUT: CHARACTER STRING "REPLY".
C    THERE ARE TWO SPECIAL CASES OF REPLY:
C    (1) IF REPLY = ' ', WHICH IS PRODUCED BY THE USER TYPING IN
C	NOTHING EXCEPT ZERO OR MORE BLANKS AND A CARRIAGE RETURN,
C	THE PROGRAM WILL RETURN CMD = ' ' AND NWD = 0.
C	NOTE THAT THIS IS AN ANOMALOUS CONDITION, AND THE CALLING
C	PROGRAM HAS THE RESPONSIBILITY OF CHECKING FOR IT AND
C	DECIDING HOW TO RESPOND TO IT.
C	THE REASON IT IS INCLUDED IS THAT IT IS VERY EASY TO
C	HIT THE CARRIAGE RETURN, AND IN PROGRAMS WITH A SINGLE FUNCTION
C	THAT IS REPEATED MANY TIMES, IT IS VERY CONVENIENT TO HAVE IT
C	PERFORMED THIS WAY.
C    (2) IF THE REPLY IS '?', AND THIS IS NOT A WORD IN CMDLIS,
C	IT IS INTERPRETED AS A REQUEST TO SEE THE
C	FULL COMMAND LIST ON THE TERMINAL.
C	THIS IS DONE, FOLLOWED BY ANOTHER PROMPT.
C
C OUTPUT PARAMETERS:
C	CMD = SUBSTRING WHICH CONTAINS THE FULL COMMAND.
C	   IN THE SPECIAL CASE OF NO RESPONSE, RETURNS ' '.
C	NWD = SERIAL NUMBER OF THE COMMAND IN THE LIST.  FOR NULL 
C	   RESPONSE, RETURNS ZERO.
C
C LOGIC STRUCTURE:
C    GET REPLY IN RESPONSE TO PROMPT
C    IF CAPSONLY IS TRUE, CAPITALIZE THE INPUT STRING.  THE ASSUMPTION
C	HERE IS THAT THE COMMAND LIST IS FULLY CAPITALIZED (AS IN
C	THE PROGRAM "PIX") BUT THE USER MAY BE USING A TERMINAL WITH
C	LOWER CASE CHARACTERS.
C    IF REPLY = ' ' (NO INPUT AT ALL FROM USER), RETURN WITH CMD = ' '
C	AND NWD = 0. (LEAVES IT UP TO THE USER TO DECIDE WHAT TO DO).
C    COMPARE REPLY WITH INPUT COMMAND LIST.
C	IF THERE IS NO MATCH, CHECK TO SEE IF THE INPUT IS A QUESTION
C	    MARK.  IF SO, TYPE THE FULL COMMAND LIST TO THE TERMINAL
C	    AND REPROMPT.  IF NOT, TYPE AN ERROR MESSAGE AND REPROMPT.
C	IF THERE IS A MULTIPLE MATCH, TYPE A MESSAGE AND START OVER.
C	IF THERE IS A SINGLE MATCH, FIND THE FULL COMMAND AND RETURN.
C
C
C
C
	CHARACTER PROMPT*(*), CMDLIS*(*), CMD*(*), REPLY*50
	LOGICAL CAPSONLY
C
4	CALL TISTRG(PROMPT,REPLY)
1	IF(CAPSONLY) CALL CAPITALIZE(REPLY)
	CALL MTCHWD(REPLY,CMDLIS,MATCH)
	IF(MATCH.EQ.-2) THEN
	  CMD = ' '
	  NWD = 0
	  RETURN
	  END IF
	IF(MATCH.EQ.-1) THEN
	  CALL TISTRG(' AMBIGUOUS WORD (NOT ENOUGH LETTERS).'//
     *     '  TRY AGAIN: ',REPLY)
	  GO TO 1
	  END IF
	IF(MATCH.EQ.0) THEN
	  IF(REPLY.EQ.'?') THEN
C	    TYPE *,'POSSIBLE COMMANDS: '//CMDLIS
	    TYPE *,CMDLIS
	    GO TO 4
	    END IF
	  CALL TISTRG(' I DON''T RECOGNIZE THIS WORD.'//
     *     ' TRY AGAIN: ',REPLY)
	  GO TO 1
	  END IF
	IF(MATCH.GT.0) THEN
	  CALL FINDWD(CMDLIS,MATCH,JA,JB)
	  CMD = CMDLIS(JA:JB)
	  NWD = MATCH
	  RETURN
	  END IF
C
	GO TO 4		!THIS SHOULD NEVER BE REACHED.
	END

C
C
	SUBROUTINE TISTRG(PROMPT,RESPONSE)
C
C THIS IS THE BASIC INPUT SUBROUTINE.
C PROMPTS THE USER BY TYPING OUT WHATEVER APPEARS IN "PROMPT",
C THEN RETURNS WHATEVER THE USER TYPES IN BEFORE THE NEXT CARRIAGE
C RETURN AS THE STRING "RESPONSE".
C
C IF NOTHING IS TYPED IN BY THE USER (I.E., IF HE JUST HITS THE RETURN)
C THEN "RESPONSE" CONTAINS A SINGLE SPACE CHARACTER (ASCII 32)
C I.E.,  RESPONSE.EQ.' '
C
	CHARACTER PROMPT*(*), RESPONSE*(*)
	TYPE 1, PROMPT
1	FORMAT(A,$)
C NOTE THAT THE $ SUPPRESSES THE CARRIAGE RETURN AFTER PROMPT IS WRITTEN
C IFF THE FIRST CHARACTER IN PROMPT IS A BLANK OR A PLUS SIGN.
	ACCEPT 2, RESPONSE
2	FORMAT(A)
	RETURN
	END

	SUBROUTINE CAPSTRG(PROMPT,RESPONSE)
C
C					J.KRISTIAN 12/30/81.
C EXTENSION OF TISTRG.  CAPITALIZES REPLY BEFORE RETURNING.
C
	CHARACTER PROMPT*(*), RESPONSE*(*)
C
	CALL TISTRG(PROMPT,RESPONSE)
	CALL CAPITALIZE(RESPONSE)
C
	RETURN
	END

	SUBROUTINE PXSTRG(PROMPT,RESPONSE)
C
C					J.KRISTIAN 12/30/81.
C EXTENSION OF TISTRG.  CAPITALIZES REPLY BEFORE RETURNING.
C ALTERNATIVE NAME FOR CAPSTRG.
C
	CHARACTER PROMPT*(*), RESPONSE*(*)
C
	CALL CAPSTRG(PROMPT,RESPONSE)
C
	RETURN
	END

	SUBROUTINE TINTGR(PROMPT,INTGR,NERROR)
C GETS A SINGLE INTEGER FROM THE USER.
C NERROR = 0 MEANS NO ERROR.
C NERROR = 1 MEANS THAT DCODEI4 WAS UNABLE TO DECODE THE INPUT
C   IN THIS CASE THE NUMBER 0 IS RETURNED TO INTGR.
	CHARACTER PROMPT*(*), INPUT*20
	NERROR = 0
	CALL TISTRG(PROMPT,INPUT)
	CALL DCODEI4(INPUT,1,INTGR,IFERR)
	IF(IFERR.NE.0) THEN
	  NERROR = 1
	  INTGR = 0
	  END IF
	RETURN
	END

	SUBROUTINE FINDWD(STRING,JWD,JA,JB)
C
C GIVEN A STRING OF CHARACTERS, FINDS THE JTH WORD (WORDS ARE DEFINED
C AS SUBSTRINGS SEPARATED BY ONE OR MORE BLANKS, COMMAS, TABS
C OR NULLS [DECIMAL ZEROS]); THIS IS THE SUBSTRING STRING(JA:JB).
C JA AND JB ARE THE RETURNED QUANTITIES.
C IF THE JTH WORD DOES NOT EXIST, RETURNS JA=JB=0.
C
C NOTE THE FOLLOWING BUG: A COMMA IS TREATED EXACTLY AS A BLANK.
C THIS IS OK EXCEPT IN THE CASE WHERE THE FIRST NON-BLANK CHARACTER
C IS A COMMA, SINCE ONE NORMALLY THINKS OF A COMMA AS A SEPARATOR
C BETWEEN WORDS.  THE BUG HAS NOT BEEN FIXED SINCE I AM NOT
C SURE HOW TO HANDLE IT.  IN THE CONTEXT, IT IS CLEARLY AN ILLEGAL
C SITUATION, AND THERE IS NO PROVISION IN THE ROUTINE FOR AN
C ERROR CODE RETURN.
C
C AFTER CALLING THIS SUBROUTINE,
C BE CAREFUL TO CHECK FOR JA=0 BEFORE TRYING TO USE
C STRING(JA:JB), SINCE THE SYSTEM WILL BLINDLY PICK UP WHATEVER
C HAPPENS TO LIE JUST TO THE LEFT OF THE RANGE OF STRING.
C
C HANDLES NEGATIVE VALUES FOR JWD WITHOUT BLOWING ANYTHING UP.
C (JUST RETURNS JA=JB=0)
C
	CHARACTER STRING*(*), NULL*1, TAB*1
	LOGICAL*1 ZERO, TABNUM
	EQUIVALENCE (NULL,ZERO), (TAB,TABNUM)
	DATA ZERO/0/, TABNUM/9/
C
	NCHAR = LEN(STRING)
	NWORD = 0
	DO 1 I=1,NCHAR
C IF CHARACTER IS A BLANK OR COMMA OR TAB, SKIP IT.
	IF(STRING(I:I).EQ.' '.OR.STRING(I:I).EQ.','.
     *   OR.STRING(I:I).EQ.NULL.OR.STRING(I:I).EQ.TAB) GO TO 1
C
C CHECK TO SEE IF CHARACTER IS THE BEGINNING OF A WORD.
C THIS IS TRUE ONLY IF THE CHARACTER IS THE FIRST IN THE STRING OR
C IS PRECEDED BY A BLANK OR A COMMA OR A TAB.
	IF(I.EQ.1) THEN
	  NWORD = 1
	  JA = 1
	 ELSE IF (STRING(I-1:I-1).EQ.' '.OR.STRING(I-1:I-1).EQ.','.
     *    OR.STRING(I-1:I-1).EQ.NULL.OR.STRING(I-1:I-1).EQ.TAB) THEN
	  NWORD = NWORD + 1
	  JA = I
	  END IF
C
C CHECK TO SEE IF CHARACTER IS THE END OF A WORD.
C THIS IS TRUE ONLY IF THE CHARACTER IS THE LAST IN THE STRING OR
C IS FOLLOWED BY A BLANK OR A COMMA OR A TAB.
	IF(I.EQ.NCHAR) THEN
	  JB = NCHAR
	 ELSE IF (STRING(I+1:I+1).EQ.' '.OR.STRING(I+1:I+1).EQ.','.
     *   OR.STRING(I+1:I+1).EQ.NULL.OR.STRING(I+1:I+1).EQ.TAB) THEN
	  JB = I
	  IF(NWORD.EQ.JWD) GO TO 2
	  END IF
1	CONTINUE
C
2	IF(NWORD.NE.JWD) JA = 0
	IF(NWORD.NE.JWD) JB = 0
	RETURN
	END

	SUBROUTINE MTCHWD(STRING,WDLIST,MATCH)
C
C DETERMINES WHETHER THE FIRST WORD IN STRING MATCHES
C (I.E., IS IDENTICALLY EQUAL TO THE INITIAL CHARACTERS OF)
C ONE OR MORE WORDS IN WDLIST.
C
C INPUT PARAMETERS: THE CHARACTER STRINGS "STRING" AND "WDLIST".
C
C OUTPUT PARAMETER:
C    MATCH (INTEGER*4) = THE RESULT OF THE SEARCH. POSSIBLE VALUES ARE:
C	-2 : STRING CONTAINS NO WORDS
C	-1 : THE FIRST WORD IN STRING MATCHES MORE THAN ONE WORD
C	     IN WDLIST.
C	 0 : THE FIRST WORD IN STRING MATCHES NO WORDS IN WDLIST
C	 n : (>0) THE FIRST WORD IN STRING MATCHES A SINGLE WORD
C	        WHOSE SERIAL NUMBER IN WDLIST IS n.
C
C	CASES SUCH AS WDLIST CONTAINING NO WORDS, OR THE FIRST WORD
C	IN STRING BEING LONGER THAN ANY WORD IN WDLIST GIVE MATCH = 0.
C
C THE CRUCIAL TEST IS THE STATEMENT IF(INDEX .....
C INDEX(C1,C2) IS A VAX CHARACTER LIBRARY FUNCTION WHICH LOOKS
C FOR THE SUBSTRING C2 IN THE STRING C1.  IF C2 APPEARS IN C1 ONE
C OR MORE TIMES, IT RETURNS THE STARTING POSITION OF THE FIRST
C APPEARANCE.  IF C2 DOES NOT APPEAR IN C1, IT RETURNS THE VALUE 0.
C
C IF THE FIRST WORD OF "STRING" IS A PRECISE MATCH WITH ONE
C OF THE WORDS IN THE LIST (IN PARTICULAR, IF IT HAS
C THE SAME NUMBER OF CHARACTERS), THEN THAT IS DEFINED 
C IMMEDIATELY AS A MATCH.  THIS MEANS
C THAT IF ONE OF THE WORDS IN THE LIST IS AN INITIAL SUBSET OF
C ANOTHER (FOR EXAMPLE, 'ABCD' AND 'ABCDEF'), THEN IF THE
C INPUT IS EXACTLY THE CORRECT WORD, IT WILL BE DECLARED
C TO BE A MATCH.  IT ALSO MEANS THAT IF THERE ARE TWO
C IDENTICAL WORDS IN THE LIST, THE FIRST ONE WILL BE
C DECLARED TO BE A MATCH.
C
	CHARACTER WDLIST*(*), STRING*(*)
C
	CALL FINDWD(STRING,1,NASTR,NBSTR)
C	 NASTR AND NBSTR ARE THE FIRST AND LAST LOCATIONS OF
C	 THE FIRST WORD IN STRING.
	IF(NASTR.EQ.0) THEN	!THERE IS NO WORD IN STRING
	  MATCH = -2
	  RETURN
	  END IF
C
	MATCH = 0
	NWORD = 0
C
1	NWORD = NWORD + 1
	CALL FINDWD(WDLIST,NWORD,NA,NB)
	IF(NA.EQ.0) RETURN	!END OF LIST
	IF(INDEX(WDLIST(NA:NB),STRING(NASTR:NBSTR)).EQ.1) THEN
C	  A MATCH HAS BEEN FOUND
	  LENGTH = NBSTR-NASTR+1
	  MATCHLENGTH = NB-NA+1
	  IF(LENGTH.EQ.MATCHLENGTH) THEN   !THE MATCH IS PRECISE.
	    MATCH = NWORD
	    RETURN
	    END IF
	  IF(MATCH.NE.0) THEN  !AN EARLIER MATCH WAS ALSO FOUND.
	    MATCH = -1
	   ELSE
	    MATCH = NWORD
	    END IF
	  END IF
	GO TO 1		!TRY NEXT WORD IN LIST
C
	END

	SUBROUTINE DCODEI4(STRING,NWD,INTEGR,IFERR)
	  ENTRY     IDCODE(STRING,NWD,INTEGR,IFERR)  !ALIAS
C
C CONVERTS A WORD CONSISTING OF A SEQUENCE OF INTEGER CHARACTERS
C INTO THE CORRESPONDING INTEGER VARIABLE.  THE WORD IS ASSUMED
C TO BE IMBEDDED IN A CHARACTER STRING.
C
C INPUT PARAMETERS:
C
C	STRING = AN ARBITRARY LENGTH CHARACTER STRING
C	NWD = THE SERIAL NUMBER (IN THE STRING) OF THE WORD 
C		TO BE DECODED.
C
C OUTPUT PARAMETERS:
C
C	INTEGR = THE DECODED INTEGER VARIABLE.
C		IF THERE IS AN ERROR IN THE DECODING, THE VALUE
C		ZERO IS RETURNED TO INTEGR.
C	IFERR = ERROR CODE, WITH THE FOLLOWING POSSIBLE VALUES:
C		0 - THERE ARE NO ERRORS.
C		1 - THE STATEMENT DECODE HAS RETURNED AN ERROR FLAG.
C		    THE TWO POSSIBILITIES IN THIS CASE ARE
C		    EITHER
C		    (A) THE WORD IS NOT A SEQUENCE OF DIGITS 
C		    OR
C		    (B) THE DECODED NUMBER IS TOO LARGE TO FIT INTO
C		    INTEGER*4.  THIS MEANS THAT IT IS EITHER BIGGER THAN
C		    2**31-1 = 2147483647 OR SMALLER THAN -2**32.
C		2 - THE WORD DOES NOT EXIST IN THE STRING.
C
C
	CHARACTER STRING*(*)
	INTEGR = 0
	IFERR = 0
C
	CALL FINDWD(STRING,NWD,NA,NB)
	IF(NA.EQ.0) THEN  ! THE WORD DOES NOT EXIST.
	  IFERR = 2
	  RETURN
	  END IF
	IF(STRING(NB:NB).EQ.'/') THEN
C		THIS IS FOR THE CASE OF A LIST ENDED BY A SLASH.
	  NB = NB-1
	  IF(NB.LT.NA) THEN ! IN THIS CASE, THE WORD IS A SINGLE SLASH.
	    IFERR = 2
	    RETURN
	    END IF
	  END IF
C
	DO 3 I=NA,NB
	IF(STRING(I:I).EQ.'A') GO TO 1
3	CONTINUE
C		THIS LOOP CORRECTS FOR A BUG IN THE STATEMENT "DECODE"
C		BY WHICH "A" IS INTERPRETED AS "10"
C
	DECODE(NB-NA+1,2,STRING(NA:NB),ERR=1) INTEGR
2	FORMAT(I)
	RETURN
C
1	IFERR = 1
	RETURN
	END
C

	SUBROUTINE DCODE2I4(STRING,I1,I2,NERROR)
C
C  DECODES A STRING, TO BE INTERPRETED AS TWO INTEGERS.
C
C INPUT VARIABLE:
C
C	STRING
C
C INPUT/OUTPUT VARIABLES:
C
C	I1, I2 = TWO INTEGER VARIABLES IN "STRING".
C		IF THE FIRST TWO WORDS IN "STRING" ARE LEGAL
C		INTEGERS, THESE ARE THE RETURNED VALUES.
C		OTHERWISE, THE INPUT VALUES ARE UNCHANGED.
C
C		(NOTE THAT IT BEHAVES DIFFERENTLY IN THIS REGARD
C		FROM ALL THE OTHER DCODE.. ROUTINES, WHICH RETURN
C		ZEROS FOR THE PARAMETERS IF THERE IS AN ERROR,
C		AND EXPECT THE USER TO KEEP UP WITH THE OLD VALUE.)
C
C OUTPUT VARIABLE:
C
C	NERROR = ERROR FLAG.  POSSIBLE VALUES:
C		0 MEANS STRING CONTAINS 2 INTEGER VARIABLES AND NO MORE.
C		  IN THIS CASE, I1 AND I2 WILL BE THE DECODED VALUES.
C		1 MEANS THAT THE FIRST WORD IS NOT AN INTEGER VARIABLE
C		  OR IS SO LARGE THAT IT OVERFLOWS THE I*4 FORMAT.
C		  IN THIS CASE, I1 AND I2 WILL CONTAIN THE INPUT VALUES.
C		2 MEANS THAT THE FIRST WORD DOES NOT EXIST; I.E., THAT
C		  THE INPUT STRING IS EMPTY.
C	
C		  1 AND 2 ARE THE SAME ERROR CODES AS DCODEI4
C
C		3 MEANS THAT THE FIRST WORD IS AN INTEGER VARIABLE BUT
C		  THE SECOND EITHER DOES NOT EXIST OR IS NOT AN INTEGER.
C		  IN THIS CASE, I1 WILL CONTAIN THE DECODED VALUE,
C		  AND I2 WILL CONTAIN THE ORIGINAL INPUT VALUE.
C		4 MEANS THAT THE FIRST TWO WORDS ARE INTEGER VARIABLES,
C		  BUT THERE EXISTS AT LEAST ONE MORE WORD IN THE STRING.
C		  IN THIS CASE, I1 AND I2 WILL BE THE DECODED VALUES.
C
C INTERNAL VARIABLES:
C
C	N1 AND N2 ARE THE DECODED VALUES OF THE INPUT STRING.
C		IF BOTH ARE OK, THEY ARE RETURNED AS I1 AND I2.
C
C
	CHARACTER STRING*(*)
C
1	CONTINUE
C
	NERROR = 0
C
	CALL DCODEI4(STRING,1,N1,IERR)
	IF(IERR.NE.0) THEN
	  NERROR = IERR    !POSSIBLE VALUES ARE 1 OR 2.
	  RETURN
	  END IF
C
	I1 = N1
C
	CALL DCODEI4(STRING,2,N2,IERR)
	IF(IERR.NE.0) THEN
	  NERROR = 3
	  RETURN
	  END IF
C
	I2 = N2
C
	CALL FINDWD(STRING,3,JA,JB)
	IF(JA.NE.0) THEN  !THERE ARE MORE THAN 2 WORDS.
	  NERROR = 4
	  END IF
C
	RETURN
	END

	SUBROUTINE DCODER4(STRING,NWD,R4VAR,IFERR)
C
C CONVERTS A WORD CONSISTING OF A SEQUENCE OF CHARACTERS
C INTO THE CORRESPONDING REAL*4 VARIABLE.
C INPUTS ARE "STRING" AND NWD, THE SERIAL NUMBER OF THE WORD 
C TO BE DECODED.
C OUTPUT IS R4VAR, THE CORRESPONDING VARIABLE.
C
C IF NWD DOES NOT EXIST IN THE STRING, OR IF IT IS NOT A SEQUENCE
C OF NUMERIC CHARACTERS, THE SUBROUTINE RETURNS IFERR = 1
C AND R4VAR = 0:  OTHERWISE RETURNS IFERR = 0
C 
C ADAPTED FROM DCODEI4 ON 5/7/80.
C
	CHARACTER STRING*(*)
	REAL*4 R4VAR
C
	R4VAR = 0.
	IFERR = 0
	CALL FINDWD(STRING,NWD,NA,NB)
	IF(NA.EQ.0) GO TO 1
C
	DO 3 I=NA,NB
	IF(STRING(I:I).EQ.'A') GO TO 1
3	CONTINUE
C THIS LOOP IS PUT IN TO BYPASS A BUG IN THE STATEMENT "DECODE"
C BY WHICH "A" IS INTERPRETED AS "10"
C
	IF(STRING(NB:NB).EQ.'/') NB = NB-1
C THIS STATEMENT IS INCLUDED TO TAKE CARE OF THE FOLKS WHO END THE
C LIST WITH A SLASH.
	IF(NB.LT.NA) GO TO 1
C
	DECODE(NB-NA+1,2,STRING(NA:NB),ERR=1) R4VAR
2	FORMAT(F15.0)
	RETURN
C
1	IFERR = 1
	RETURN
	END

	SUBROUTINE DCODER8(STRING,NWD,R8VAR,IFERR)
C
C CONVERTS A WORD CONSISTING OF A SEQUENCE OF CHARACTERS
C INTO THE CORRESPONDING REAL*8 VARIABLE.
C INPUTS ARE "STRING" AND NWD, THE SERIAL NUMBER OF THE WORD 
C TO BE DECODED.
C OUTPUT IS R8VAR, THE CORRESPONDING VARIABLE, AND THE ERROR CODE IFERR.
C
C IF NWD DOES NOT EXIST IN THE STRING, OR IF IT IS NOT A SEQUENCE
C OF NUMERIC CHARACTERS, THE SUBROUTINE RETURNS IFERR = 1
C AND R8VAR = 0:  OTHERWISE RETURNS IFERR = 0
C 
C ADAPTED FROM DCODER4 ON 10/20/80.
C CHANGED FORMAT 2 FROM F15.0 TO F20.0 2/82.
C
	CHARACTER STRING*(*)
	REAL*8 R8VAR
C
	R8VAR = 0.D0
	IFERR = 0
	CALL FINDWD(STRING,NWD,NA,NB)
	IF(NA.EQ.0) GO TO 1
C
	DO 3 I=NA,NB
	IF(STRING(I:I).EQ.'A') GO TO 1
3	CONTINUE
C THIS LOOP IS PUT IN TO BYPASS A BUG IN THE STATEMENT "DECODE"
C BY WHICH "A" IS INTERPRETED AS "10"
C
	IF(STRING(NB:NB).EQ.'/') NB = NB-1
C THIS STATEMENT IS INCLUDED TO TAKE CARE OF THE FOLKS WHO END THE
C LIST WITH A SLASH.
	IF(NB.LT.NA) GO TO 1
C
	DECODE(NB-NA+1,2,STRING(NA:NB),ERR=1) R8VAR
2	FORMAT(F20.0)
	RETURN
C
1	IFERR = 1
	RETURN
	END

	SUBROUTINE NEWR4(PROMPT,R4VAR)
C
C LIKE NEWI4 BUT FOR REAL*4 VARIABLE.
C
C RECOGNIZES THREE VALID REPLIES IN RESPONSE TO "PROMPT":
C  (1) IF REPLY IS A LEGITIMATE VARIABLE, IT IS RETURNED AS "R4VAR".
C  (2) IF REPLY IS A CARRIAGE RETURN OR ONE OR MORE BLANKS,
C      RETURNS WITHOUT CHANGING THE VALUE OF "R4VAR".
C	THE CURRENT VALUE IS ALSO TYPED TO THE TERMINAL.
C  (3) IF REPLY IS A QUESTION MARK ("?"), TYPES OUT THE PRESENT VALUE
C      OF "R4VAR", AND PROMPTS FOR A POSSIBLE NEW VALUE OF "R4VAR"
C
C CHANGED TO PASS THROUGH TO NEWR4X 4/17/84.
C
	CHARACTER PROMPT*(*), REPLY*20
C
1	CALL NEWR4X(PROMPT,R4VAR,IERR,REPLY)
	IF(IERR.NE.0) GO TO 1
C
	RETURN
	END

	SUBROUTINE NEWR4X(PROMPT,R4VAR,IERR,REPLY)
C
C					J. KRISTIAN 4/17/84
C
C GENERALIZATION OF NEWR4.
C DOES NOT FORCE USER TO INPUT AN R4 VARIABLE.
C INTENDED TO HANDLE SPECIAL CASES WHERE OTHER REPLIES MAY BE VALID.
C
C INPUT VARIABLES:
C
C	PROMPT (CHARACTER STRING)
C
C OUTPUT VARIABLES:
C
C	R4VAR
C	IERR = ERROR RETURN FROM DCODER4.
C	REPLY
C
C RECOGNIZES THREE VALID REPLIES IN RESPONSE TO "PROMPT":
C  (1) IF REPLY IS A LEGITIMATE VARIABLE, IT IS RETURNED AS "R4VAR".
C  (2) IF REPLY IS A CARRIAGE RETURN OR ONE OR MORE BLANKS,
C      RETURNS WITHOUT CHANGING THE VALUE OF "R4VAR".
C	THE CURRENT VALUE IS ALSO TYPED TO THE TERMINAL.
C  (3) IF REPLY IS A QUESTION MARK ("?"), TYPES OUT THE PRESENT VALUE
C      OF "R4VAR", AND PROMPTS FOR A POSSIBLE NEW VALUE OF "R4VAR"
C  (4) IF REPLY IS ANYTHING ELSE, RETURNS WITHOUT CHANGING INPUT VALUE
C	OF R4VAR.
C
	CHARACTER PROMPT*(*), REPLY*(*)
C
	IERR = 0
C
1	CALL TISTRG(PROMPT,REPLY)
C
2	IF(REPLY.EQ.' ') THEN
	  TYPE *, R4VAR
	  RETURN
	  END IF
C
	IF(REPLY.EQ.'?') THEN
	  TYPE *, 'PRESENT VALUE :',R4VAR
	  CALL TISTRG('  NEW VALUE :  ',REPLY)
	  GO TO 2
	  END IF
C
	CALL DCODER4(REPLY,1,VAR,IERR)
	IF(IERR.EQ.0) R4VAR = VAR
	
	RETURN
C
	END
C

	SUBROUTINE NEWR8(PROMPT,R8VAR)
C
C LIKE NEWR8 BUT FOR REAL*8 VARIABLE.
C
C RECOGNIZES THREE VALID REPLYS IN RESPONSE TO "PROMPT":
C  (1) IF REPLY IS A LEGITIMATE VARIABLE, IT IS RETURNED AS "R8VAR".
C  (2) IF REPLY IS A CARRIAGE RETURN OR ONE OR MORE BLANKS,
C      RETURNS WITHOUT CHANGING THE VALUE OF "R8VAR".
C	THE CURRENT VALUE IS TYPED TO THE TERMINAL BEFORE RETURNING.
C  (3) IF REPLY IS A QUESTION MARK ("?"), TYPES OUT THE PRESENT VALUE
C      OF "R8VAR", AND PROMPTS FOR A POSSIBLE NEW VALUE OF "R8VAR"
C
C  CHANGED LENGTH OF REPLY FROM 20 TO 25 2/82.
C
	CHARACTER PROMPT*(*), REPLY*25
	REAL*8 R8VAR,VAR
C
1	CALL TISTRG(PROMPT,REPLY)
C
2	IF(REPLY.EQ.' ') THEN
	  TYPE *, R8VAR
	  RETURN
	  END IF
C
	IF(REPLY.EQ.'?') THEN
	TYPE *, 'PRESENT VALUE :',R8VAR
	CALL TISTRG('  NEW VALUE :  ',REPLY)
	GO TO 2
	END IF
C
	CALL DCODER8(REPLY,1,VAR,IERR)
	IF(IERR.NE.0) GO TO 1
	R8VAR = VAR
	RETURN
C
	END
C

	SUBROUTINE NEW2R4(PROMPT,VAR1,VAR2)
C
C LIKE NEWR4, BUT FOR TWO VARIABLES.
C
	PARAMETER NREPLY = 20
	CHARACTER PROMPT*(*), REPLY*(NREPLY)
C
1	CALL TISTRG(PROMPT,REPLY)
C
2	IF(REPLY.EQ.' ') THEN
	  TYPE *, VAR1, VAR2
	  RETURN
	  END IF
C
	IF(REPLY.EQ.'?') THEN
	TYPE *,'PRESENT VALUES:',VAR1,VAR2
	CALL TISTRG(' NEW VALUES?  ',REPLY)
	GO TO 2
	END IF
C
	CALL DCODER4(REPLY,1,VAR1,IERR)
	IF(IERR.NE.0) GO TO 1
	CALL DCODER4(REPLY,2,VAR2,IERR)
	IF(IERR.NE.0) THEN
3	  TYPE *,'1ST NUMBER =',VAR1
	  CALL TISTRG(' WHAT IS 2ND NUMBER? ',REPLY)
	  CALL DCODER4(REPLY,1,VAR2,IERR)
	  IF(IERR.NE.0) GO TO 3
	  END IF
	RETURN
	END

	SUBROUTINE INTLIST(STRING,J,NMAX,N,LIST)
C GIVEN A STRING OF CHARACTERS ASSUMED TO CONTAIN A LIST OF INTEGERS
C IMBEDDED IN IT, BEGINNING WITH THE JTH WORD OF THE STRING.
C THERE CAN BE A INDEFINITE NUMBER OF INTEGERS IN THE LIST, UP TO NMAX.
C
C THE SUBROUTINE WILL DECODE THE LIST AND RETURN IT IN THE INTEGER
C ARRAY "LIST".  N (0.LE.N.LE.NMAX) IS THE NUMBER OF VALUES ACTUALLY 
C FOUND IN THE LIST.
C IF N = 0 (I.E., IF NO NUMBERS ARE FOUND AT THE START OF THE LIST),
C THEN THE ARRAY "LIST" IS LEFT UNCHANGED BY THIS SUBROUTINE.
C
	CHARACTER STRING*(*)
	INTEGER*4 LIST(NMAX)
	N=0
	DO 1 I=J,J+NMAX-1
	CALL DCODEI4(STRING,I,INTEG,IFERR)
	IF(IFERR.NE.0) RETURN
	N = N+1
1	LIST(N) = INTEG
	RETURN
	END

	SUBROUTINE INTLIST2(STRING,J,NMAX,N,LIST,NERROR)
C
C   *****************  NOT YET COMPLETED OR TESTED  ***************
C
C					J. KRISTIAN 3/24/83.
C
C GENERALIZATION OF INTLIST WHICH RETURNS AN ERROR CODE.
C
C GIVEN A STRING OF CHARACTERS ASSUMED TO CONTAIN A LIST OF INTEGERS
C IMBEDDED IN IT, BEGINNING WITH THE JTH WORD OF THE STRING.
C THERE CAN BE A INDEFINITE NUMBER OF INTEGERS IN THE LIST, UP TO NMAX.
C
C THE SUBROUTINE WILL DECODE THE LIST AND RETURN IT IN THE INTEGER
C ARRAY "LIST".  N (0.LE.N.LE.NMAX) IS THE NUMBER OF VALUES ACTUALLY 
C FOUND IN THE LIST.
C THE PROCESS IS TRUNCATED IF (1) THE END OF THE STRING IS REACHED;
C (2) A WORD IN THE STRING IS REACHED WHICH IS EITHER NOT AN INTEGER
C OR IS TOO LARGE TO BE DECODED AS INTEGER*4; (3) NMAX INTEGERS ARE
C DECODED, ALTHOUGH MORE WORDS MAY EXIST IN THE LIST.
C
C IF N = 0 (I.E., IF NO NUMBERS ARE FOUND AT THE START OF THE LIST),
C THEN THE ARRAY "LIST" IS LEFT UNCHANGED BY THIS SUBROUTINE.
C
C INPUT VARIABLES:
C
C	STRING = A CHARACTER STRING OF UNDEFINED LENGTH.
C	J (INTEGER*4) : THE INTEGER LIST IS ASSUMED TO START WITH
C		THE Jth WORD OF "STRING".
C	NMAX = THE MAXIMUM NUMBER OF INTEGERS IN THE STRING
C		(= THE DIMENSION OF LIST)
C
C OUTPUT VARIABLES:
C
C	N = THE ACTUAL NUMBER OF INTEGERS FOUND IN THE LIST.
C	LIST (INTEGER*4) = AN ARRAY CONTAINING THE DECODED LIST
C		OF INTEGERS.
C	NERROR : 
C
C		0 - NO ERROR
C		1 - THE Jth WORD DOES NOT EXIST. I.E., THE STRING IS
C		    EMPTY OVER THE REQUESTED RANGE.
C		    IN THIS CASE, THE INPUT VALUES OF "N" AND THE INPUT
C		    VALUES IN "LIST" ARE UNCHANGED.
C		    (A SPECIAL CASE IS J = 1,
C		    IN WHICH CASE THE ENTIRE STRING IS EMPTY).
C		2 -
C	OTHER POSSIBLE ERRORS:
C
C		THE JTH WORD HAS A BAD CHARACTER OR IS TOO LARGE
C		  (IFERR = 1)
C	        THE ITH WORD (I>J) DITTO.
C		THERE ARE MORE THAN NMAX WORDS IN THE LIST.
C
	CHARACTER STRING*(*)
	INTEGER*4 LIST(NMAX)
C
	N=0
	DO 1 I=J,J+NMAX-1
	CALL DCODEI4(STRING,I,INTEG,IFERR)
	IF(IFERR.NE.0) THEN
	  IF(IFERR.EQ.2 .AND. I.EQ.J) THEN
	    NERROR = 1
	    RETURN
	    END IF
	  END IF
	N = N+1
1	LIST(N) = INTEG
	RETURN
	END

	SUBROUTINE PIXLIST(LIST,MAXNUMBER,NUMBER,NERROR)
C
C					J. KRISTIAN 12/81
C
C A GENERALIZATION OF FRLIST.  IF NO NUMBERS ARE TYPED IN, 
C RETURNS THE OLD LIST AND NUMBER COUNT AND NERROR = 1, INSTEAD
C OF A NUMBER COUNT OF ZERO AS DONE BY FRLIST.
C
C THE REAL TEST FOR AN ERROR IS TO LOOK AT "NUMBER".  IF THERE IS
C NOTHING IN THE LIST, THIS WILL BE EQUAL TO ZERO.
C
C NOTE THAT ANY PROMPT DESIRED MUST BE TYPED BY THE CALLING PROGRAM
C BEFORE PIXLIST IS CALLED; PIXLIST DOES NOT PROMPT FOR THE LIST.
C
	INTEGER*4 LIST(MAXNUMBER)
C
	CALL FRLIST(LIST,MAXNUMBER,NEWNUMBER)
	IF(NEWNUMBER.EQ.0) THEN
	  NERROR = 1
	 ELSE
	  NERROR = 0
	  NUMBER = NEWNUMBER
	  END IF
C
	RETURN
	END

      SUBROUTINE FRLIST(LISTOUT,MAXOUT,NOUT)
C
C					J. KRISTIAN
C THIS IS FILIST MODIFIED FOR USE ON VAX 6/79.
C
C TRANSLATES A LIST OF NUMBERS ASSUMED TO BE ALL POSITIVE.
C IF A NEGATIVE NUMBER IS ENCOUNTERED IN THE LIST FOLLOWING A
C POSITIVE NUMBER (THUS: N1 -N2) IT IS INTERPRETED AS FOLLOWS:
C	IF N2.GT.N1 IT IS TAKEN TO MEAN N1 THROUGH N2, INCLUSIVE;
C	IF N2.LT.N1 THE LIST IS TERMINATED AFTER N2.
C
C NOTE THAT ANY PROMPT DESIRED MUST BE TYPED BY THE CALLING PROGRAM
C BEFORE FRLIST IS CALLED; FRLIST DOES NOT PROMPT FOR THE LIST.
C
C INPUT VARIABLES: 
C
C	MAXOUT = UPPER LIMIT TO THE NUMBER OF ITEMS IN THE LIST 
C		(THE DIMENSION OF THE ARRAY LISTOUT).
C
C OUTPUT VARIABLES:
C
C	LISTOUT = AN ARRAY CONTAINING THE TRANSLATED LIST OF NUMBERS.
C	NOUT = THE NUMBER OF ITEMS ACTUALLY IN THE LIST	(AS DECODED).
C	THE LIST OF ITEMS IS RETURNED IN THE ORDER GIVEN (NOT SORTED).
C
C
C INTERNAL VARIABLES:
C
C LISTIN IS THE LIST AS READ, INCLUDING NEGATIVE NUMBERS.  IT NOW
C HAS A DIMENSION MAXIN = 100.
C ITEMS IN THE INPUT LIST ARE SEPARATED BY ONE OR MORE BLANKS, COMMAS,
C OR COMBINATIONS OF BLANKS AND COMMAS.
C
C
C
CHANGED 8/14/78 TO USE INTLIST FOR INPUT, SO THAT THE LIST DOES NOT 
C NECESSARILY HAVE TO BE TERMINATED BY A SLASH.
C
CHANGED 8/17/79: XPANLIST WAS MADE A SEPARATE SUBROUTINE, SPLIT OFF
C AND PUT INTO TIOPCKG.
C
C 11/12/81: ADDED PASS OF INPUT STRING THROUGH SUBROUTINE SPMINUS,
C SO THAT A STRING WHICH HAS ITEMS LIKE "3-9", WITH NO SPACE OR COMMA
C BEFORE THE MINUS SIGN, WILL BE INTERPRETED CORRECTLY.
C
C
	PARAMETER MAXIN = 100
	DIMENSION LISTOUT(MAXOUT),LISTIN(MAXIN)
	CHARACTER INPUT*200
	DO 1 I=1,MAXIN
1	LISTIN(I)=0
	CALL TISTRG(' ',INPUT)
	CALL SPMINUS(INPUT)
	CALL INTLIST(INPUT,1,MAXIN,NIN,LISTIN)
	IF(NIN.LE.0) THEN
	  NOUT = 0
	  RETURN
	  END IF
	CALL XPANLIST(LISTIN,NIN,MAXOUT,LISTOUT,NOUT)
	RETURN
	END
C

      SUBROUTINE FRLIST2(LISTOUT,MAXOUT,NOUT,NERROR)
C
C  **************  NOT YET FINISHED OR TESTED  ******************
C
C  **  INSTEAD OF CHECKING INPUT HERE (E.G., SEEING IF EMPTY),
C  **  SHOULD PROBABLY FINISH INTLIST2 AND USE IT TO THE MAXIMUM.
C
C
C
C					J. KRISTIAN  3/24/83.
C
C GENERALIZATION OF FRLIST.
C DOES CHECKING ON INPUT, AND RETURNS ERROR CODES.
C
C THIS IS FILIST MODIFIED FOR USE ON VAX 6/79.
C
C TRANSLATES A LIST OF NUMBERS ASSUMED TO BE ALL POSITIVE.
C IF A NEGATIVE NUMBER IS ENCOUNTERED IN THE LIST FOLLOWING A
C POSITIVE NUMBER (THUS: N1 -N2) IT IS INTERPRETED AS FOLLOWS:
C	IF N2.GT.N1 IT IS TAKEN TO MEAN N1 THROUGH N2, INCLUSIVE;
C	IF N2.LT.N1 THE LIST IS TERMINATED AFTER N2.
C
C INPUT VARIABLES: 
C
C	MAXOUT = UPPER LIMIT TO THE NUMBER OF ITEMS IN THE LIST 
C		(THE DIMENSION OF THE ARRAY LISTOUT).
C
C OUTPUT VARIABLES:
C
C	LISTOUT = AN ARRAY CONTAINING THE TRANSLATED LIST OF NUMBERS.
C	NOUT = THE NUMBER OF ITEMS ACTUALLY IN THE LIST	(AS DECODED).
C	 THE LIST OF ITEMS IS RETURNED IN THE ORDER GIVEN (NOT SORTED).
C	NERROR:  ERROR RETURN CODE.  INDICATES SOME PROBLEM WITH THE
C		INPUT STRING.  HAS THE FOLLOWING VALUES;
C
C		1 : NO INPUT (THE INPUT STRING IS EMPTY).
C
C
C INTERNAL VARIABLES:
C
C LISTIN IS THE LIST AS READ, INCLUDING NEGATIVE NUMBERS.  IT NOW
C HAS A DIMENSION MAXIN = 100.
C ITEMS IN THE INPUT LIST ARE SEPARATED BY ONE OR MORE BLANKS, COMMAS,
C OR COMBINATIONS OF BLANKS AND COMMAS.
C
C
C
CHANGED 8/14/78 TO USE INTLIST FOR INPUT, SO THAT THE LIST DOES NOT 
C NECESSARILY HAVE TO BE TERMINATED BY A SLASH.
C
CHANGED 8/17/79: XPANLIST WAS MADE A SEPARATE SUBROUTINE, SPLIT OFF
C AND PUT INTO TIOPCKG.
C
C 11/12/81: ADDED PASS OF INPUT STRING THROUGH SUBROUTINE SPMINUS,
C SO THAT A STRING WHICH HAS ITEMS LIKE "3-9", WITH NO SPACE OR COMMA
C BEFORE THE MINUS SIGN, WILL BE INTERPRETED CORRECTLY.
C
C
	PARAMETER MAXIN = 100
	DIMENSION LISTOUT(MAXOUT),LISTIN(MAXIN)
	CHARACTER INPUT*200
C
	NERROR = 0
C
	CALL TISTRG(' ',INPUT)
	IF (INPUT.EQ.' ') THEN
	  NERROR = 1
	  RETURN
	  END IF
C
	DO 1 I=1,MAXIN
1	LISTIN(I)=0
C
	CALL SPMINUS(INPUT)
	CALL INTLIST(INPUT,1,MAXIN,NIN,LISTIN)
	IF(NIN.LE.0) THEN
	  NOUT = 0
	  RETURN
	  END IF
	CALL XPANLIST(LISTIN,NIN,MAXOUT,LISTOUT,NOUT)
	RETURN
	END
C

	SUBROUTINE SPMINUS(STRING)
C
C					J. KRISTIAN 11/81.
C
C IF STRING CONTAINS MINUS SIGNS NOT PRECEDED BY A SPACE OR A COMMA,
C INSERTS A SPACE BEFORE EACH ONE.
C
C USED BY FRLIST TO MAKE THE INPUT REQUIREMENT LESS RIGID.
C
	CHARACTER STRING*(*)
C
	CALL COUNTWDS(STRING,NWDS,JA,JB)
	IF(JA.EQ.0) RETURN	!EMPTY STRING
C
	J = JA
1	CONTINUE
	J = J+1
	IF(J.LE.JB) THEN
	  IF ((STRING(J:J).EQ.'-').
     *     AND.(STRING(J-1:J-1).NE.' ').
     *     AND.(STRING(J-1:J-1).NE.','))  THEN
	    STRING(J+1:JB+1) = STRING(J:JB)
	    STRING(J:J) = ' '
	    JB = JB + 1
	    END IF
	  GO TO 1
	  END IF
C
	RETURN
	END

	SUBROUTINE XPANLIST(LISTIN,NIN,MAXOUT,LISTOUT,NOUT)
C GIVEN A LIST OF NON-ZERO INTEGERS, WHICH IS ASSUMED TO BE AN
C ABBREVIATED FORM OF A LIST OF POSITIVE INTEGERS.  THE SUBROUTINE
C INTERPRETS THE LIST AS FOLLOWS:
C IF THE INPUT LIST CONTAINS A SUCCESSIVE PAIR OF ENTRIES
C N1 -N2, THIS IS TAKEN TO MEAN THE POSITIVE NUMBERS
C N1 THROUGH N2, INCLUSIVE.  (IF N2.LT.N1, THE LIST WILL BE TRUNCATED,
C WITH N1 BEING THE LAST ENTRY.)
C THE LIST THUS EXPANDED IS RETURNED IN THE ARRAY LISTOUT.
C NOUT IS THE NUMBER OF ENTRIES RETURNED IN LISTOUT.
C IF NOUT TRIES TO BE LARGER THAN MAXOUT, THE OUTPUT LIST WILL BE 
C TRUNCATED AT MAXOUT ENTRIES.
C NOTE THAT NO SORTING IS DONE ON THE LIST.
	DIMENSION LISTIN(NIN),LISTOUT(MAXOUT)
11    NOUT=0
	IF(NIN.EQ.0) RETURN
      NEGFLG=0
C
      DO 2 I=1,100
      IF(NEGFLG.EQ.1)GO TO 7
      NA=LISTIN(I)
      IF(NA.LE.0)RETURN
C QUITS IF ANY ENTRY = 0 OR LIST STARTS WITH NEGATIVE NUMBER,
C OR LIST CONTAINS TWO NEGATIVE NUMBERS IN A ROW.
      NB=LISTIN(I+1)
      IF(NB)3,4,5
5     NOUT=NOUT+1
	IF(NOUT.GT.MAXOUT) GO TO 8 !TOO MANY ENTRIES IN INPUT LIST
      LISTOUT(NOUT)=NA
      GO TO 2
4     NOUT=NOUT+1
	IF(NOUT.GT.MAXOUT) GO TO 8 !TOO MANY ENTRIES IN INPUT LIST
      LISTOUT(NOUT)=LISTIN(I)
      RETURN
3     NB=-NB
      DO 6 K=NA,NB
      NOUT=NOUT+1
	IF(NOUT.GT.MAXOUT) GO TO 8 !TOO MANY ENTRIES IN INPUT LIST
6     LISTOUT(NOUT)=K
      NEGFLG=1
      GO TO 2
7     NEGFLG=0
2     CONTINUE
      RETURN
C
8	NOUT = MAXOUT 
	RETURN
      END

	SUBROUTINE GETI4(PROMPT,INTGR)
	ENTRY ONEINT(PROMPT,INTGR)
C FORCES A SINGLE INTEGER FROM THE USER BY REPEATED CALLS
C TO TINTGR.
	CHARACTER PROMPT*(*)
1	CALL TINTGR(PROMPT,INTGR,NERROR)
	IF(NERROR.NE.0) GO TO 1
	RETURN
	END

	SUBROUTINE RANGEI4(PROMPT,IMIN,IMAX,I1,I2)
	  ENTRY    TWOINTS(PROMPT,IMIN,IMAX,I1,I2)
C
C ******* 9/16/82: CHANGED TO PASS THROUGH TO SUBROUTINE DCODEI4RANGE.
C ******* THIS IS PREPARATORY TO ADDING CURSOR INPUTS AS
C ******* ANOTHER POSSIBILITY.
C
C
C FORCES TWO INTEGERS (I1 AND I2) TO BE ENTERED FROM THE TERMINAL.
C BOTH INTEGERS MUST BE IN THE RANGE IMIN THROUGH IMAX, INCLUSIVE.
C
C FOR DETAILS, SEE THE COMMENTS IN THE SUBROUTINE DCODEI4RANGE.
C
C
	PARAMETER NREPLY = 20
	CHARACTER PROMPT*(*), REPLY*(NREPLY)
C
1	CALL TISTRG(PROMPT,REPLY)
C
6	IF(REPLY.EQ.'?') THEN
	TYPE *,'PRESENT VALUES:',I1,I2
	CALL TISTRG(' NEW VALUES?  ',REPLY)
	GO TO 6
	END IF
C
	CALL DCODEI4RANGE(REPLY,IMIN,IMAX,I1,I2,NERROR)
C
	IF(NERROR.EQ.0) RETURN
C
	IF(NERROR.EQ.1) THEN
	  TYPE *, 'RANGE OF INPUT LIES OUTSIDE LIMITS',IMIN,' TO',IMAX
	  TYPE *, 'TRY AGAIN.'
	  GO TO 1
	  END IF
C
	IF(NERROR.EQ.2) THEN
	  TYPE *, 'I CAN''T DECODE THE INPUT AS TWO INTEGERS.'
	  TYPE *, 'TRY AGAIN.'
	  GO TO 1
	  END IF
C
	IF(NERROR.LT.0 .OR. NERROR.GT.2) THEN  !SHOULDN'T HAPPEN.
	  TYPE *,'UNKNOWN ERROR RETURN FROM DCODEI4RANGE: CODE =',NERROR
	  RETURN
	  END IF
C
	END

	SUBROUTINE RANGEI4OLD(PROMPT,IMIN,IMAX,I1,I2)
	  ENTRY    TWOINTSOLD(PROMPT,IMIN,IMAX,I1,I2)
C
C ******* 9/82: CHANGED TO PASS THROUGH TO SUBROUTINE
C ******* DCODEI4RANGE, PREPARATORY TO ALLOWING INPUT FROM
C ******* CURSORS.
C
C
C FORCES TWO INTEGERS (I1 AND I2) TO BE ENTERED FROM THE TERMINAL.
C BOTH INTEGERS MUST BE IN THE RANGE IMIN THROUGH IMAX, INCLUSIVE.
C
C WILL ALSO DECODE THE ENTRIES ">N" AND "<N" [OR "> N", ETC.].
C THE (INPUT) VALUES IMIN AND IMAX ARE USED IN THIS DECODING. THUS,
C  ">N" IS DECODED AS I1 = N+1 AND I2 = IMAX
C  "<N" IS DECODED AS I1 = IMIN AND I2 = N-1.
C
C NOTE THAT THE LARGEST AND SMALLEST PERMISSIBLE VALUES OF VAX I*4
C NUMBERS ARE +/- 2**31-1 = 2147483647.  IF THESE ARE INPUT FOR
C IMAX AND IMIN, THEY WILL COVER THE RANGE OF PERMISSIBLE NUMBERS.
C
C GENERALIZED 5/12/80 TO ALLOW THE POSSIBILITY OF LEAVING THE VALUES 
C OF I1 AND I2 UNCHANGED. (FOR EXAMPLE, SEE SUBROUTINES GETI4 AND NEWI4)
C NOW RECOGNIZES TWO OTHER VALID INPUTS:
C  (1) IF INPUT = CARRIAGE RETURN OR ALL BLANKS, RETURNS LEAVING I1 AND
C      I2 UNCHANGED.  CURRENT VALUES ARE TYPED TO THE TERMINAL.
C  (2) IF INPUT = '?', TYPES CURRENT VALUES OF I1, I2, PROMPTS
C      FOR NEW VALUES.
C
	PARAMETER NREPLY = 20
	CHARACTER PROMPT*(*), REPLY*(NREPLY)
C
1	CALL TISTRG(PROMPT,REPLY)
C
6	IF(REPLY.EQ.' ') THEN
	  TYPE *, I1, I2
	  RETURN
	  END IF
C
	IF(REPLY.EQ.'?') THEN
	TYPE *,'PRESENT VALUES:',I1,I2
	CALL TISTRG(' NEW VALUES?  ',REPLY)
	GO TO 6
	END IF
C
	CALL FINDWD(REPLY,1,JA,JB)
	IF(JA.EQ.0) GO TO 1
C
	IF(REPLY(JA:JA).EQ.'>') THEN
	CALL DCODEI4(REPLY(JA+1:NREPLY),1,IX,IERR)
	IF(IERR.EQ.0) THEN
	  IF(IX.GT.IMAX-1) THEN
	    TYPE *,'NUMBER TOO BIG: MUST BE LESS THAN ',IMAX
	    GO TO 1
	    END IF
	  I1 = IX+1
	  IF(I1.LT.IMIN) I1 = IMIN
	  I2 = IMAX
	  RETURN
	 ELSE
2	  CALL TISTRG(' BIGGER THAN WHAT NUMBER? ',REPLY)
	  CALL DCODEI4(REPLY,1,IX,IERR)
	  IF(IERR.NE.0) GO TO 2
	  I1 = IX+1
	  IF(I1.LT.IMIN) I1 = IMIN
	  I2 = IMAX
	  RETURN
	  END IF
	END IF
C
	IF(REPLY(JA:JA).EQ.'<') THEN
	CALL DCODEI4(REPLY(JA+1:NREPLY),1,IX,IERR)
	IF(IERR.EQ.0) THEN
	  IF(IX.LT.IMIN+1) THEN
	    TYPE *,'NUMBER TOO SMALL: MUST BE BIGGER THAN',IMIN
	    GO TO 1
	    END IF
	  I1 = IMIN
	  I2 = IX-1
	  IF(I2.GT.IMAX) I2 = IMAX
	  RETURN
	 ELSE
4	  CALL TISTRG(' LESS THAN WHAT NUMBER? ',REPLY)
	  CALL DCODEI4(REPLY,1,IX,IERR)
	  IF(IERR.NE.0) GO TO 4
	  I1 = IMIN
	  I2 = IX-1
	  IF(I2.GT.IMAX) I2 = IMAX
	  RETURN
	  END IF
	END IF
C
	CALL DCODEI4(REPLY,1,I1,IERR)
	IF(IERR.NE.0) GO TO 1
	IF(I1.LT.IMIN.OR.I1.GT.IMAX) THEN
	  TYPE *, 'FIRST NUMBER LIES OUTSIDE THE RANGE',IMIN,' TO',IMAX
	  GO TO 1
	  END IF
	CALL DCODEI4(REPLY,2,I2,IERR)
	IF(I2.LT.IMIN.OR.I2.GT.IMAX) THEN
	  TYPE *, 'SECOND NUMBER LIES OUTSIDE THE RANGE',IMIN,' TO',IMAX
	  GO TO 5
	  END IF
3	IF(IERR.EQ.0) RETURN
	TYPE *, '1ST NUMBER =',I1
5	CALL GETI4(' WHAT IS 2ND NUMBER? ',I2)
	IF(I2.LT.IMIN.OR.I2.GT.IMAX) THEN
	  TYPE *,'NUMBER LIES OUTSIDE THE RANGE',IMIN,' TO',IMAX
	  GO TO 5
	  END IF
	END

	SUBROUTINE GET2I4(PROMPT,I1,I2)
C
C FORCES TWO INTEGERS (I1 AND I2) TO BE ENTERED FROM THE TERMINAL.
C
	PARAMETER NREPLY = 20
	CHARACTER PROMPT*(*), REPLY*(NREPLY)
C
1	CALL TISTRG(PROMPT,REPLY)
	CALL FINDWD(REPLY,1,JA,JB)
	IF(JA.EQ.0) GO TO 1
C
C
	CALL DCODEI4(REPLY,1,I1,IERR)
	IF(IERR.NE.0) GO TO 1
	CALL DCODEI4(REPLY,2,I2,IERR)
	IF(IERR.NE.0) THEN
	  TYPE *,'1ST NUMBER =',I1
	  CALL GETI4(' WHAT IS 2ND NUMBER? ',I2)
	  END IF
	RETURN
	END

	SUBROUTINE NEW2I4(PROMPT,I1,I2)
C
C LIKE NEWI4, BUT FOR TWO INTEGERS.
C
C CHANGED 8/8/82 BY SPLITTING OFF MOST OF ROUTINE INTO
C NEW2I4DCODE (Q.V.).
C
	PARAMETER NREPLY = 20
	CHARACTER PROMPT*(*), REPLY*(NREPLY)
C
1	CALL TISTRG(PROMPT,REPLY)
C
	CALL NEW2I4DCODE(PROMPT,REPLY,I1,I2)
C
	RETURN
	END

	SUBROUTINE NEW2I4DCODE(PROMPT,REPLY,I1,I2)
C
C				J. KRISTIAN  8/8/82.
C  THIS IS NEW2I4 WITH THE INITIAL PROMPT REMOVED.
C  IT WAS CREATED IN ORDER TO DEAL WITH THE CASE WHERE
C  ANOTHER KIND OF RESPONSE MIGHT BE MEANINGFUL BESIDES
C  TWO INTEGERS.  SPECIFICALLY, IT WAS SET UP TO HANDLE
C  THE CASE "A" FOR AUTOMATIC IN PIX'S IMAGE DISPLAY
C  (SUBROUTINE DISPTI2), BUT IT CLEARLY CAN BE USED
C  FOR SIMILAR SITUATIONS.
C  IT IS A REAL KLUGE, AND SHOULD BE FIXED UP SOMETIME.
C
C  NOTE THAT BOTH PROMPT AND REPLY ARE INPUT VARIABLES.
C
C
	CHARACTER PROMPT*(*), REPLY*(*)
C
1	CONTINUE
C
2	IF(REPLY.EQ.' ') THEN
	  TYPE *,I1,I2
	  RETURN
	  END IF
C
	IF(REPLY.EQ.'?') THEN
	TYPE *,'PRESENT VALUES:',I1,I2
	CALL TISTRG(' NEW VALUES?  ',REPLY)
	GO TO 2
	END IF
C
	CALL DCODE2I4(REPLY,I1,I2,NERROR)
	IF(NERROR.EQ.0) RETURN
	IF(NERROR.EQ.1) THEN
	  CALL TISTRG(PROMPT,REPLY)
	  GO TO 1
	  END IF
	IF(NERROR.EQ.3) THEN
	  TYPE *,'1ST NUMBER =',I1
	  CALL GETI4(' WHAT IS 2ND NUMBER? ',I2)
	  END IF
C	NOTE THAT NO ACTION IS TAKEN IF NERROR = 4
C	(MORE WORDS AFTER THE TWO INTEGERS).
C
	RETURN
	END

	SUBROUTINE COUNTWDS(STRING,NWDS,JA,JB)
C
C COUNTS THE NUMBER OF WORDS IN A STRING AND GIVES ITS SIGNIFICANT
C LIMITS (IGNORING LEADING AND TRAILING BLANKS).
C RETURNS ARE NWDS = NUMBER OF WORDS IN THE INPUT STRING,
C JA = LOCATION IN "STRING" OF THE FIRST CHARACTER OF THE FIRST WORD,
C JB =     "     "     "    "   "   LAST     "       "  "  LAST   "    .
C IF NWDS = 0, ALSO RETURNS JA=JB=0
C
	CHARACTER STRING*(*)
C
	NWDS = 0
	CALL FINDWD(STRING,1,JA,JB)
	IF(JA.EQ.0) RETURN
C
	DO 1 NWD = 1,LEN(STRING)
	CALL FINDWD(STRING,NWD,J1,J2)
	IF(J1.EQ.0) RETURN
	JB = J2
1	NWDS = NWD
C
	END

	SUBROUTINE NEWI4(PROMPT,NUMBER)
C
C GENERALIZATION OF GETI4.  FOR OPTIONAL CHANGE OF "NUMBER".
C
C RECOGNIZES THREE VALID INPUTS IN RESPONSE TO "PROMPT":
C  (1) IF INPUT IS A LEGITIMATE INTEGER, IT IS RETURNED AS "NUMBER".
C  (2) IF INPUT IS A CARRIAGE RETURN OR ONE OR MORE BLANKS,
C      RETURNS WITHOUT CHANGING THE VALUE OF "NUMBER", 
C	AFTER TYPING THE CURRENT VALUE OF "NUMBER" ON THE TERMINAL.
C  (3) IF INPUT IS A QUESTION MARK ("?"), TYPES OUT THE INPUT VALUE
C      OF "NUMBER", AND PROMPTS FOR A POSSIBLE NEW VALUE OF "NUMBER"
C
	CHARACTER PROMPT*(*), INPUT*20
C
1	CALL TISTRG(PROMPT,INPUT)
C
2	IF(INPUT.EQ.' ') THEN
	  TYPE *,NUMBER
	  RETURN
	  END IF
C
	IF(INPUT.EQ.'?') THEN
	TYPE *, 'PRESENT VALUE :',NUMBER
	CALL TISTRG('  NEW VALUE :  ',INPUT)
	GO TO 2
	END IF
C
	CALL DCODEI4(INPUT,1,NEWNUM,IERR)
	IF(IERR.NE.0) GO TO 1
	NUMBER = NEWNUM
	RETURN
C
	END

	SUBROUTINE MERGSTRG(STRING1,STRING2,MERGED)
C
C COMBINES TWO STRINGS INTO ONE, WITH TRAILING BLANKS STRIPPED
C OFF THE FIRST STRING (BUT LEADING BLANKS ARE LEFT IN BOTH STRINGS).
C
C  MERGED MAY BE THE SAME AS STRING1, BUT MUST BE
C  DISTINCT FROM STRING2.
C
	CHARACTER STRING1*(*), STRING2*(*), MERGED*(*)
C
	CALL COUNTWDS(STRING1,NWDS1,JA1,JB1)
	IF(NWDS1.EQ.0) MERGED = ' '
	IF(NWDS1.GT.0) MERGED = STRING1(1:JB1)
	CALL COUNTWDS(STRING2,NWDS2,JA2,JB2)
	IF (NWDS2.EQ.0) RETURN
	IF(NWDS1.EQ.0) THEN
	  MERGED = STRING2(1:JB2)
	 ELSE
	  MERGED = MERGED(1:JB1)//STRING2(1:JB2)
	  END IF
C
	RETURN
	END

	SUBROUTINE NEWXYLIM(NXDIM,NYDIM,NXA,NXB,NYA,NYB)
C
C					J. KRISTIAN  1982
C
C ******  NOTE (9/82).  RANGEI4 HAS BEEN CHANGED IN SUCH A WAY
C ******  AS TO MAKE ALL OF THE ERROR CHECKING IN THIS ROUTINE
C ******  REDUNDANT.  THE ROUTINE COULD CONSIST ENTIRELY OF
C ******  STATEMENTS 1 AND 2
C
C
C ASSUMING A TWO DIMENSIONAL ARRAY WITH X RUNNING FROM 1 TO
C NXDIM AND Y RUNNING FROM 1 TO NYDIM, GETS TWO PAIR OF NUMBERS
C FROM THE USER WHICH ARE INTERPRETED TO BE X AND Y BOUNDS
C WITHIN THE ARRAY.
C
C INPUT PARAMETERS: NXDIM,NYDIM
C
C OUTPUT PARAMETERS:
C	NXA,NXB = LOWER AND UPPER X BOUNDS.
C		THE PROGRAM CHECKS TO MAKE SURE THAT NXA AND NXB
C		LIE WITHIN THE ARRAY, AND THAT NXB.GE.NXA,
C		AND FORCES RESPONSES THAT SATISFY THESE CONDITIONS.
C	NYA,NYB = LOWER AND UPPER Y BOUNDS.
C
C	IF THE USER RESPONDS WITH A CARRIAGE RETURN FOR EITHER SET
C	OF BOUNDS, THE EXISTING VALUES ARE UNCHANGED.
C
C	IF HE RESPONDS WITH A QUESTION MARK, THE CURRENT VALUES
C	ARE DISPLAYED AND HE IS GIVEN A CHANCE TO CHANGE THEM.
C
C	ALSO RECOGNIZES INPUTS SUCH AS "> 10"  (SEE RANGEI4).
C
1	CALL RANGEI4(' LOWER AND UPPER X BOUNDS: ',1,NXDIM,NXA,NXB)
	IF(NXA.LT.1) THEN
	  TYPE *,'ERROR: LOWER X BOUND TOO SMALL.  MINIMUM = 1.'
	  GO TO 1
	  END IF
	IF(NXB.GT.NXDIM) THEN
	  TYPE *,'ERROR: UPPER X BOUND TOO LARGE.  MAXIMUM =',NXDIM
	  GO TO 1
	  END IF
	IF(NXB.LT.NXA) THEN
	  TYPE *,'ERROR: UPPER BOUND IS SMALLER THAN LOWER BOUND.'
	  GO TO 1
	  END IF
C
2	CALL RANGEI4(' LOWER AND UPPER Y BOUNDS: ',1,NYDIM,NYA,NYB)
	IF(NYA.LT.1) THEN
	  TYPE *,'ERROR: LOWER Y BOUND TOO SMALL.  MINIMUM = 1.'
	  GO TO 2
	  END IF
	IF(NYB.GT.NYDIM) THEN
	  TYPE *,'ERROR: UPPER Y BOUND TOO LARGE.  MAXIMUM =',NYDIM
	  GO TO 2
	  END IF
	IF(NYB.LT.NYA) THEN
	  TYPE *,'ERROR: UPPER BOUND IS SMALLER THAN LOWER BOUND.'
	  GO TO 2
	  END IF
C
	RETURN
	END

	SUBROUTINE CAPITALIZE(STRING)
C
C					J. KRISTIAN 12/81
C CONVERTS ANY LOWER CASE LETTERS IN STRING TO UPPER CASE,
C ASSUMING ASCII CHARACTERS.
C
	CHARACTER STRING*(*), CHAR*1
	LOGICAL*1 NUMBER
	EQUIVALENCE (CHAR,NUMBER)
C
	DO 1 I = 1,LEN(STRING)
	CHAR = STRING(I:I)
	IF(NUMBER.GE.'61'X.AND.NUMBER.LE.'7A'X) THEN
	  NUMBER = NUMBER - '20'X
	  STRING(I:I) = CHAR
	  END IF
1	CONTINUE
C
	RETURN
	END

	SUBROUTINE DCODEI4RANGE(STRING,IMIN,IMAX,I1,I2,NERROR)
C
C					J.  KRISTIAN  9/82.
C
C
C ADAPTED FROM RANGEI4.
C CONTAINS MOST OF THE WORKING PARTS FOR THAT SUBROUTINE, WHICH
C WAS CHANGED TO PASS THROUGH TO THIS ONE.
C
C INPUT VARIABLES:
C
C	STRING = CHARACTER STRING TO BE DECODED AS TWO INTEGERS
C		IN THE RANGE IMIN TO IMAX, INCLUSIVE.
C		LEGAL FORMS OF STRING ARE AS FOLLOWS:
C
C		   TWO INTEGERS.
C		   '>N' OR '> N', WHERE N.LE.IMAX-1
C		   '<N' OR '< N', WHERE N.GE.IMIN+1
C		   '?'.  IN THIS CASE THE PRESENT VALUES OF
C		         I1 AND I2 ARE TYPED AND NEW VALUES
C		         PROMPTED FOR.
C		   ' '.  IN THIS CASE, THE PRESENT VALUES IF
C		         I1 AND I2 ARE TYPED AND RETURNED.
C
C	IMIN, IMAX = MINIMUM AND MAXIMUM VALUES PERMITTED FOR THE
C		TWO INTEGERS.
C
C		IF THE RANGE OF THE TWO NUMBERS DEFINED BY THE INPUT
C		STRING IS LARGER THAN THE RANGE DEFINED BY IMIN
C		AND IMAX, THE NUMBERS RETURNED IN I1 AND I2 WILL
C		DEFINE ONLY THE OVERLAP REGION.  FOR EXAMPLE IF
C		IMIN=20, IMAX=40 AND THE INPUT STRING IS '10 30',
C		THE ROUTINE WILL RETURN I1=20 AND I2=30.
C		IN THIS CASE, AN ERROR CODE OF NERROR = -1 IS
C		RETURNED (SOFT ERROR).
C
C
C OUTPUT VARIABLES:
C
C	I1, I2 = DECODED VALUES FOR THE TWO NUMBERS, WITH I1.LE.I2
C		NOTE THAT IN THE INPUT STRING, THE NUMBERS MAY BE IN
C		THE OTHER ORDER, WITH THE LARGER FIRST.
C
C	NERROR = ERROR CODE AS FOLLOWS:
C	       -1: STRING DECODED AS INTEGERS, BUT THEIR RANGE
C		   EXTENDS BEYOND THE LIMITS IMIN-IMAX.
C		   IN THIS CASE, THE VALUES RETURNED FOR I1 AND I2
C		   DEFINE THE OVERLAP REGION. (SOFT ERROR)
C		0: STRING SUCCESSFULLY DECODED.
C		1: STRING DECODED AS INTEGERS, BUT THEIR RANGE
C		   DOES NOT OVERLAP THE RANGE IMIN-IMAX, AS FOLLOWS;
C			STRING = '>N', N.GE.IMAX, OR
C			STRING = '<N', N.LE.IMIN, OR
C			STRING = TWO INTEGERS, SMALLER>MAX OR LARGER<MIN
C		2: STRING NOT DECODABLE AS TWO INTEGERS (WRONG
C		   CHARACTERS IN STRING).
C		3: IMIN > IMAX.
C
C		IF NERROR.GT.0, THE INPUT VALUES OF I1 AND I2 ARE
C		NOT CHANGED.
C
C
C NOTE THAT THE LARGEST AND SMALLEST PERMISSIBLE VALUES OF VAX I*4
C NUMBERS ARE +/- 2**31-1 = 2147483647.  IF THESE ARE INPUT FOR
C IMAX AND IMIN, THEY WILL COVER THE RANGE OF PERMISSIBLE NUMBERS.
C
C
	CHARACTER STRING*(*)
C
C
	NSTRING = LEN(STRING)
	NERROR = 0
C
	IF(IMIN.GT.IMAX) THEN
	  NERROR = 3
	  RETURN
	  END IF
C
6	IF(STRING.EQ.' ') THEN
	  TYPE *, I1, I2
	  RETURN
	  END IF
C
	IF(STRING.EQ.'?') THEN
	TYPE *,'PRESENT VALUES:',I1,I2
	CALL TISTRG(' NEW VALUES?  ',STRING)
	GO TO 6
	END IF
C
	CALL FINDWD(STRING,1,JA,JB)
	IF(JA.EQ.0) THEN
	  NERROR = 2
	  RETURN
	  END IF
C
	IF(STRING(JA:JA).EQ.'>') THEN
	CALL DCODEI4(STRING(JA+1:NSTRING),1,IX,IERR)
	IF(IERR.EQ.0) THEN
	  IF(IX.GT.IMAX-1) THEN
	    NERROR = 1
	    RETURN
	    END IF
	  I1 = IX+1
	  IF(I1.LT.IMIN) THEN
	    NERROR = -1
	    I1 = IMIN
	    END IF
	  I2 = IMAX
	  RETURN
	 ELSE
	  NERROR = 2
	  RETURN
	  END IF
	END IF
C
	IF(STRING(JA:JA).EQ.'<') THEN
	CALL DCODEI4(STRING(JA+1:NSTRING),1,IX,IERR)
	IF(IERR.EQ.0) THEN
	  IF(IX.LT.IMIN+1) THEN
	    NERROR = 1
	    RETURN
	    END IF
	  I1 = IMIN
	  I2 = IX-1
	  IF(I2.GT.IMAX) THEN
	    NERROR = -1
	    I2 = IMAX
	    END IF
	  RETURN
	 ELSE
	  NERROR = 2
	  RETURN
	  END IF
	END IF
C
	CALL DCODEI4(STRING,1,IA,IERR1)
	CALL DCODEI4(STRING,2,IB,IERR2)
	IF(IERR1.NE.0 .OR. IERR2.NE.0) THEN
	  NERROR = 2
	  RETURN
	  END IF
	IF(IA.GT.IB) THEN  !INVERT THE ORDER
	  I = IA
	  IA = IB
	  IB = I
	  END IF
	IF(IB.LT.IMIN.OR.IA.GT.IMAX) THEN
	  NERROR = 1
	  RETURN
	  END IF
C
	I1 = MAX(IA,IMIN)
	I2 = MIN(IB,IMAX)
C
	IF(I1.LT.IMIN .OR. I2.GT.IMAX) NERROR = -1
C
	RETURN
	END

	SUBROUTINE YESORNO(PROMPT,RESPONSE)
C
C					J.KRISTIAN 3/83.
C
C FORCES USER TO RETURN ONE OF 3 RESPONSES:
C 	"Y(ES)", "N(O)" OR " " (CARRIAGE RETURN)
C
	CHARACTER PROMPT*(*), RESPONSE*(*), INPUT*10, YES*3, NO*2
	DATA YES/'YES'/, NO/'NO'/
C
1	CALL CAPSTRG(PROMPT,INPUT)
C
	CALL COUNTWDS(INPUT,NWDS,JA,JB)
	IF(NWDS.GT.1) THEN
	  TYPE *,' INPUT MUST BE "Y(ES)", "N(O)" OR A CARRIAGE RETURN.'
	  GO TO 1
	  END IF
C
	IF(INPUT(JA:JB).NE.YES(1:JB-JA+1)
     *   .AND. INPUT(JA:JB).NE.NO(1:JB-JA+1)
     *   .AND. NWDS.NE.0) THEN
	  TYPE *,' INPUT MUST BE "Y(ES)", "N(O)" OR A CARRIAGE RETURN.'
	  GO TO 1
	  END IF
C
	RESPONSE = INPUT(JA:JB)
C
	RETURN
	END

	SUBROUTINE BELL
C TYPES THE BELL CHARACTER TO THE TERMINAL.
	CHARACTER BELLCHAR*1
	DATA BELLCHAR/'07'X/
	TYPE *,BELLCHAR
	END
