        SUBROUTINE ConnectVS(CC)
!Finds the coordinates of connected sets of atoms and presents them as a
! CRYM Atom file.  Not rigorous, but will always work if the
!maximum bond length (d1) isn't too long compared to the shortest cell
!edge times half the sine of the smaller related interaxial
!angle. The parameter d0 (say 0.5) is for a search for implausibly close
!or symmetry-superposed atoms. 

!Input: For008; Output: COORD cards in For009; Echo of input, Record of
!distances found less than d1, and Full list of atoms with
!final flags and fragment numbers in For006.

!Put a key atom of the first fragment or molecule first in the input
!COORD cards. (Other fragments, if any, will get
!successively higher NF's.  The position of the first fragment will
!be decided by your choice of its key atom; the other frag-
!ments will be determined by the first atom of each to occur in the
!atom list.)
!The COORD cards, like the cell card and the symop cards, are standard
!CRYM's. Flag values: 0,
!virgin atom; 1, primer	atom or atom found as JJ,KK and copied to 1,kk;
! 2, atom that has completed its search (as J,K) for virgin
!neighbors; 3, `duplicate' found out and rejected. 
!CONNECT COURTESTY V. SCHOMAKER.
! INCORPORATED INTO CRYM 1/25/88 BY KMS.
        Parameter nad=300, nsd=40, rad=57.29578
! nad: No.-atoms dim.; nsd: No.-symops dim.
        Integer F(nad)                                  !Control Flag
        CHARACTER*16 NFILE
        COMMON/CONSYM/IYES  ! TO MAKE SURE SYM HAS BEEN RUN
        COMMON/FNAME/NFILE,NAMLEN  ! FOR OPENING CRD FILE
        Character*4 At(nad)
        character*80 card80
        CHARACTER*3 AF(NAD)
        CHARACTER*37 JUNK(NAD) ! COORD CARD JUNK
        CHARACTER*20 REM(NAD)
        Dimension x(nsd,nad), y(nsd,nad),z(nsd,nad),Q(3,3,nsd),w(3,nsd)
        Dimension NNF(nad),CC(20)
        NFILE(NAMLEN+1:NAMLEN+4)='.CRD'
        OPEN(UNIT=19,NAME=NFILE,FORM='FORMATTED',TYPE='NEW')
        IF(IYES.EQ.0)THEN
          WRITE(3,901)
 901      FORMAT(//' MUST RUN SYM BEFORE CONNECT.'//)
          RETURN
        ENDIF
        CLOSE(UNIT=16,DISP='SAVE')
        OPEN(UNIT=16,TYPE='OLD',FORM='FORMATTED')
        DECODE(80,6,CC)D0,D1
        Read(16,1) a,b,c,al,be,ga !CELL CARD - READ FROM UNIT 16
        WRITE(3,201)
        Write(3,*)' D0 (MINIMUM DISTANCE) = ',D0,' D1 (MAXIMUM',
     1' DISTANCE = ',d1
        al = 2.*b*c*cos(al/Rad)
        be = 2.*a*c*cos(be/Rad)
        ga = 2.*a*b*cos(ga/Rad)
        a = a*a
        b = b*b
        c = c*c
!  First transformation to list is identity transformation!

        ns=1
        do i=1,3
          do j=1,3
            q(i,j,ns)=0.0
          enddo
          w(i,ns)=0.0
        enddo
        q(1,1,1)=1.0
        q(2,2,1)=1.0
        q(3,3,1)=1.0

        na=1
9       read(16,109,end=7)card80
        decode(80,101,card80,ERR=10)At(na),AF(na),x(1,na),y(1,na),
     1 z(1,na),junk(na)  ! read Coord cards FROM UNIT 16
        IF(NA.GT.NAD)GOTO 98
        na=na+1
        goto 9
10      na=na-1
        DO 14 J=1,NA
14      REM(J)='                    '
        NS=2
        decode(80,2,card80,err=5)(((q(j,k,ns),k=1,3),w(j,ns),j=1,3))
        ns=ns+1
11      Read(16,2,END=5) (((Q(J,K,NS),K=1,3),w(J,ns),J=1,3)) 
        IF(NS.GT.NSD)GOTO 99
        NS=NS+1
        GOTO 11
7       na=na-1
        DO 8 J=1,NA
8       REM(J)='                    '
        NS=2
! Read symops, including Lattice-centerings FROM UNIT 16
5       ns=ns-1
        WRITE(3,203)
        Do 18 J=2,NS              !APPLY THE SYMOPS
          Do 18 K=1,NA
           x(J,K)=Q(1,1,J)*x(1,K)+Q(1,2,J)*y(1,K)+Q(1,3,J)*z(1,K)+w(1,J)
           y(J,K)=Q(2,1,J)*x(1,K)+Q(2,2,J)*y(1,K)+Q(2,3,J)*z(1,K)+w(2,J)
18         z(J,K)=Q(3,1,J)*x(1,K)+Q(3,2,J)*y(1,K)+Q(3,3,J)*z(1,K)+w(3,J)
        F(1)=1
        NF=1
20      Do K=1,NA      !The main job: For all atoms 1,K try for F=1
            If(F(K).eq.1) Then              
              nnf(k)=nf
              Do KK=1,NA 
! Having found one, search over all atoms for 'zero'
! distances, and for distances d0<d<d1, marking the
! former F=3 and the latter F=1, as transformed to 'symop 0'
                if(kk.ne.k.and.(f(kk).eq.0.or.f(kk).eq.1))then
                  Do JJ=1,NS 
                    xr=xx(x(1,K),x(JJ,KK))
                    yr=xx(y(1,K),y(JJ,KK))
                    zr=xx(z(1,K),z(JJ,KK))
! Find the atom equivalent to JJ,KK by an integral
! lattice translation that is nearest to 1,K
                    dd=d((xr-x(1,K)),(yr-y(1,K)),(zr-z(1,K)),a,b,c,
     1 al,be,ga)
                    If(dd.le.d0) Then
                        F(KK)=3
                        REM(KK)=' OVERLAPS '//AT(K)
! Mark the `duplicate'.  (On jj=1 for f(kk)=1, by sometimes on jj>1 for
! f(kk)=0).
                        goto 22
                    Elseif(f(kk).eq.0.and.dd.le.d1) then
                      nnf(kk)=nf
                      F(KK) =1
! Atom JJ,KK found.  Atom KK moved and marked.
                      x(1,KK)=xr
                      y(1,KK)=yr
                      z(1,KK)=zr
                      goto 22
                    elseif(f(kk).eq.1.and.dd.le.d1)then
22                    write(3,104)at(k),at(kk),dd,jj,f(kk)
! atom 1, atom 2, d, last previous symop of atom 2, and flag.
                    Endif
                  Enddo
                Endif
              Enddo
              F(K)=2  ! atom 1,k is done.
              Goto 20 ! necessary to get repeated passes thru whole list
            Endif
        Enddo                     !The main job is finished.
        nf=nf+1
        do kkk=1,na
           if(f(kkk).eq.0)then  ! no atom is finally omitted from output
              f(kkk)=1           !just because it isn't linked to an
              goto 20            !input '1' atom.
            endif
        enddo
        WRITE(3,202)
        DO L=1,NF
          Do K=1,NA
            IF(NNF(K).EQ.L)THEN
              If(F(K).eq.2) Write(19,102) At(K),af(k),x(1,K),y(1,K),
     1  Z(1,K),junk(k)   ! the new coord file
              write(3,103)at(k),af(k),x(1,k),y(1,k),z(1,k),f(k),nnf(k),
     1 REM(K)  ! RECORD OF ALL ATOMS
             ENDIF
           ENDDO
        Enddo
      WRITE(3,204)
      RETURN
98    WRITE(3,981)NAD
981   FORMAT(' This program is dimensioned for a maximum of ',
     1 i3,' Atoms.',/'  You have exceeded this number, please have ',
     2'program modified.')
      STOP
99      WRITE(3,991)NSD
991   FORMAT(' This program is dimensioned for a maximum of ',
     1i2,' Symmetry operations.,
     +'/,'  You have exceeded this number, please have ',
     2'program modified.')
      STOP
1       Format(6x,6F8.4)
!This write loop could be deleted.
2       Format(6x,12F6.3)
3       format('CELL',2X,6F8.4)
6      FORMAT(8X,2F5.2)
101     Format(6x,A4,2x,a3,3F9.6,a)
102     Format('COORD ',A4,2x,a3,3f9.6,a)
103     Format(1X,'COORD ',a4,2x,a3,3F9.6,2i4,A)
104     format(6x,a,a,f9.4,2i4)
109     format(a80)
201    FORMAT('1',' CONNECT'//)
202    FORMAT(//' FINAL ATOM LIST:'//'   ATOM',13X,'X',8X,'Y',8X,'Z',
     1 5X,'FLAG',2X,'FRAG.#'/)
203    FORMAT(//' DISTANCE LOG:'/)
204    FORMAT(//' SEE CRD FILE FOR FINAL COORDINATE LIST.'/)
        END
!  defining 2 functions:
        Function xx(x,y)
        xx = Nint(x-y) + y
!Nint(a)=[a + .5*sign(a)]; [x]: the integer whose sign is sign(x)
!and whose magnitude is the largest integer that does not exceed the
!magnitude of x.
        End
        Function d(x,y,z,a,b,c,al,be,ga)
        d= Sqrt(x*x*a+y*y*b+z*z*c+y*z*al+z*x*be+x*y*ga)
        End
