	SUBROUTINE  GCONSC (ZE, MX,MY, IA,IB, JA,JB, Z0, PL2)
C-----------------------------------------------------------------------
C PGPLOT: Draw a single contour.  This routine is called by PGCONT, but
C may be called directly by the user.
C
C Arguments:
C 
C ZE (real array dimension MX,MY, input): the array of function values.
C MX,MY (integer, input): actual declared dimension of Z(*,*).
C IA,IB (integer, input): inclusive range of the first index of ZE to be
C	contoured.
C JA,JB (integer, input): inclusive range of the second index of ZE to 
C	be contoured.
C Z0 (real, input): the contour level sought.
C PL2 (the name of a subroutine declared EXTERNAL in the calling 
C	routine): this routine is called by PGCONSC to do all graphical
C	output. The calling sequence is CALL PL2(K,X,Y,Z) where Z is
C	the contour level, (X,Y) are the coordinates of a point (in the
C	inclusive range I1<X<I2, J1<Y<J2, and if K is 0, the routine is
C	to move then pen to (X,Y); if K is 1, it is to draw a line from
C	the current position to (X,Y).
C 
C NOTE:  the intervals (IA,IB) and (JA,JB) must not exceed the
C dimensions of internal array EMPTY(*,*,*) below.  These are currently
C set at 50.
C
C (7-Feb-1983)
C 28-Aug-1984 Attempt to improve behavior on plateaus [TJP].
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER  MAXEMX, MAXEMY
	PARAMETER (MAXEMX=50, MAXEMY=50)
	EXTERNAL         PL2
	INTEGER          I, I0, IA, IAM1, IARG, IARG1, IARG2, IB
	INTEGER          II, INDEX, IOFF, IX, IXPMX, J, J0
	INTEGER          JA, JAM1, JARG, JB
	INTEGER          JJ, JOFF, K, K0, L, MX, MY
C 
C     IMPLEMENTATION NOTE:  CONSIDERABLE SPACE MAY BE SAVED FOR
C     EMPTY(*,*,*) IF SHORT LOGICAL TYPES (E.G. LOGICAL*1) ARE AVAILABLE
C 
	LOGICAL*1        EMPTY(2,MAXEMX,MAXEMY), FOUND, NOTIN1, NOTIN2
	REAL             XS, YS, Z0, Z11, Z12, Z21, Z22, ZARG
	REAL             ZE(*), ZM1, ZM2
	REAL             ZP, ZP1, ZP2, ZS
	INTEGER          I1, I2, I3, J1, J2, J3
	REAL             Z1, Z2, Z3
	COMMON /PGCON1/  I1, J1, I2, J2, I3, J3, Z1, Z2, Z3
C
	INDEX(JARG,IARG) = IARG + MX*(JARG-1)
	ZP(IARG1,IARG2) = FLOAT(IARG1)-ZS*FLOAT(IARG2-IARG1)
C 
C		Check for errors.
C 
	IF ( (IB-IA) .GT. MAXEMX .OR.  (JB-JA) .GT. MAXEMY ) THEN
	    CALL GRWARN('GCONSC - array index range exceeds built-in '//
     1		        ' limit of 50')
	    RETURN
	END IF
C 
C		Scan the array cross-section, marking triangles which
C		contain part of the requested contour.  EMPTY(K,I,J) is
C		.TRUE. if triangle K (1=upper,2=lower) in the box with
C		lower-left corner at (I,J) does ***NOT*** contain the
C		contour.
C 
	II = MAX(IA,IB-1)
	JJ = MAX(JA,JB-1)
	IAM1 = IA-1
	JAM1 = JA-1
	FOUND = .FALSE.
        DO J = JA,JJ
	    IX = INDEX(J,IA)
	    JOFF = J-JAM1
	    DO I = IA,II
		IXPMX = IX+MX
		IOFF = I-IAM1
		Z11 = ZE(IX)
		Z12 = ZE(IXPMX)
		Z21 = ZE(IX+1)
		Z22 = ZE(IXPMX+1)
		ZP1 = MAX(Z11,Z12,Z21)
		ZM1 = MIN(Z11,Z12,Z21)
		ZP2 = MAX(Z12,Z21,Z22)
		ZM2 = MIN(Z12,Z21,Z22)
*		NOTIN1 = (ZP1 .LT. Z0) .OR. (ZM1 .GT. Z0)
*		NOTIN2 = (ZP2 .LT. Z0) .OR. (ZM2 .GT. Z0)
		NOTIN1 = (ZP1 .LT. Z0) .OR. (ZM1 .GE. Z0)
		NOTIN2 = (ZP2 .LT. Z0) .OR. (ZM2 .GE. Z0)
		FOUND = FOUND .OR. (.NOT.NOTIN1) .OR. (.NOT.NOTIN2)
		EMPTY(1,IOFF,JOFF) = NOTIN1
		EMPTY(2,IOFF,JOFF) = NOTIN2
		IX = IX+1
	    END DO
	END DO
	IF (.NOT.FOUND) RETURN
C
C		Contour intersects array.  Search array for a cell
C		which includes the contour, then start following the
C		contour.
C
	DO 80 J = JA,JJ
	    JOFF = J-JAM1
            DO 70 I = IA,II
                     DO 60 K = 1,2
                     IOFF = I-IAM1
                     IF (EMPTY(K,IOFF,JOFF)) GO TO 70
C 
C                    WE HAVE A CONTOUR IN TRIANGLE (K,IOFF,JOFF).
C                    FOLLOW IT THROUGH THE MESH, MARKING TRIANGLES
C                    AS VISITED WHEN THE CONTOUR PASSES THROUGH THEM.
C 
                     CALL GCONIT (I,J,K)
                     Z1 = ZE(INDEX(J1,I1)) - Z0
                     Z2 = ZE(INDEX(J2,I2)) - Z0
                     Z3 = ZE(INDEX(J3,I3)) - Z0
                     IF (Z1*Z2 .GT. 0.0) CALL GCONXV (1,3)
                     IF (Z1*Z3 .GT. 0.0) CALL GCONXV (1,2)
                     CALL GCONSA
                          DO 50 L = 1,2
                          ZS = 0.0
                          IF (Z2 .NE. Z1) ZS = Z1/(Z2 - Z1)
                          CALL PL2 (0,ZP(I1,I2),ZP(J1,J2),Z0)
   40                     CALL GCONNC
                          ZS = 0.0
                          IF (Z2 .NE. Z1) ZS = Z1/(Z2 - Z1)
                          CALL PL2 (1,ZP(I1,I2),ZP(J1,J2),Z0)
                          IF (I3 .LT. IA) GO TO 50
                          IF (I3 .GT. IB) GO TO 50
                          IF (J3 .LT. JA) GO TO 50
                          IF (J3 .GT. JB) GO TO 50
                          I0 = MIN(I1,I2,I3)-IAM1
                          J0 = MIN(J1,J2,J3)-JAM1
                          K0 = MOD(I1+I2+I3,3)
                          IF (EMPTY(K0,I0,J0)) GO TO 50
                          EMPTY(K0,I0,J0) = .TRUE.
                          Z3 = ZE(INDEX(J3,I3)) - Z0
                          GO TO 40
   50                     CALL GCONRE
   60                CONTINUE
   70           CONTINUE
   80      CONTINUE
	RETURN
	END
	SUBROUTINE  GCONIT (I,J,K)
C-----------------------------------------------------------------------
C PGPLOT (internal routine): Initial triangle. (Support routine for
C PGCONSC.)
C
C Arguments:
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	INTEGER          I, J, K, M1, M2, M3
	REAL             Z1, Z2, Z3
	COMMON /PGCON1/  M1(2), M2(2), M3(2), Z1, Z2, Z3
C
	M1(1) = I+K-1
	M1(2) = J+K-1
	M2(1) = I+1
	M2(2) = J
	M3(1) = I
	M3(2) = J+1
	RETURN
	END
	SUBROUTINE  GCONXV (I,J)
C-----------------------------------------------------------------------
C PGPLOT (internal routine): exchange the Ith and Jth vectors in
C common (support routine for PGCONSC).
C
C Arguments:
C
C I,J (integer, input): the vectors to be exchanged (1,2 or 3).
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER          I, J, MM, N1, N2
	REAL             T, Z
	COMMON /PGCON1/  MM(2,3),     Z(3)
C
	N1 = MM(1,I)
	N2 = MM(2,I)
	MM(1,I) = MM(1,J)
	MM(2,I) = MM(2,J)
	MM(1,J) = N1
	MM(2,J) = N2
	T = Z(I)
	Z(I) = Z(J)
	Z(J) = T
	RETURN
	END
	SUBROUTINE  GCONSA
C-----------------------------------------------------------------------
C PGPLOT (internal routine): Save the initial point of the contour.
C (Support routine for PGCONSC.)
C
C Arguments: none.
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER          M, N
	REAL             W, Z
	COMMON /PGCON1/  M(6), Z(3)
	COMMON /PGCON2/  N(6), W(3)
C
	N(1) = M(1)
	N(2) = M(2)
	N(3) = M(5)
	N(4) = M(6)
	N(5) = M(3)
	N(6) = M(4)
	W(1) = Z(1)
	W(2) = Z(3)
	W(3) = Z(2)
	RETURN
	END
	SUBROUTINE  GCONNC
C-----------------------------------------------------------------------
C PGPLOT (internal routine): Next, constant (support routine for
C PGCONSC). Select P3 to define the next triangle along a constant
C contour. Points P1 and P2 of the new triangle will be points of the
C old triangle, while P3 will be a new point gotten by reflection of
C the missing point in the opposite edge.
C
C Arguments: none.
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER          I, J, M1, M2, M3
	REAL             Z, Z1, Z2, Z3
	COMMON /PGCON1/  M1(2), M2(2), M3(2), Z1, Z2, Z3
C
	IF (Z1*Z3 .LT. 0.0) THEN
	    I = M1(1)-M2(1)+M3(1)
	    J = M1(2)-M2(2)+M3(2)
	    Z = Z3
	    Z3 = Z2
	    Z2 = Z
	    M2(1) = M3(1)
	    M2(2) = M3(2)
	    M3(1) = I
	    M3(2) = J
	ELSE IF (Z2*Z3 .LT. 0.0) THEN
	    I = -M1(1)+M2(1)+M3(1)
	    J = -M1(2)+M2(2)+M3(2)
	    Z = Z3
	    Z3 = Z1
	    Z1 = Z
	    M1(1) = M3(1)
	    M1(2) = M3(2)
	    M3(1) = I
	    M3(2) = J
	END IF
	RETURN
	END
	SUBROUTINE  GCONRE
C-----------------------------------------------------------------------
C PGPLOT (internal routine): Restore the initial point of the contour.
C (Support routine for PGCONSC.)
C
C Arguments: none.
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER          N, M
	REAL             W, Z
	COMMON /PGCON1/  M(6), Z(3)
      	COMMON /PGCON2/  N(6), W(3)
C
	M(1) = N(1)
	M(2) = N(2)
	M(3) = N(3)
	M(4) = N(4)
	M(5) = N(5)
	M(6) = N(6)
	Z(1) = W(1)
	Z(2) = W(2)
	Z(3) = W(3)
	RETURN
	END
	SUBROUTINE GCP (K,X,Y,Z)
C-----------------------------------------------------------------------
C PGPLOT (internal routine): Draw one contour segment (for use by
C PGCONSC). 
C
C Arguments:
C
C K (input, integer): if K=0, move the pen to (X,Y); if K=1, draw
C	a line from the current position to (X,Y); otherwise
C	do nothing.
C X (input, real): X world-coordinate of end point.
C Y (input, real): Y world-coordinate of end point.
C Z (input, real): the value of the contour level, not used by PGCP at
C	the moment.
C
C (7-Feb-1983)
C-----------------------------------------------------------------------
	IMPLICIT NONE
      REAL TRANS
       COMMON/TRMAT/TRANS(6)
	INTEGER  K
	REAL     X,XX,Y,YY,Z
C
	XX = TRANS(1) + TRANS(2)*X + TRANS(3)*Y
	YY = TRANS(4) + TRANS(5)*X + TRANS(6)*Y
	IF (K.EQ.1) THEN
	    CALL GRLINA(XX,YY)
	ELSE IF (K.EQ.0) THEN
	    CALL GRMOVA(XX,YY)
	END IF
	RETURN
	END
