	SUBROUTINE GCONT (A,IDIM,JDIM,I1,I2,J1,J2,C,NC,TR)
C-----------------------------------------------------------------------
C PGPLOT: Draw a contour map of an array.  The map is truncated if
C necessary at the boundaries of the viewport.  Each contour line
C is drawn with the current line attributes (color index, style, and
C width); except that if argument NC is positive (see below), the line
C style is set by PGCONT to 1 (solid) for positive contours or 2
C (dashed) for negative contours.
C
C Arguments:
C
C A (input, real array): data array.
C IDIM (input, integer): first dimension of A.
C JDIM (input, integer): second dimension of A.
C I1,I2 (input, integers): range of first index to be contoured
C	(inclusive).
C J1,J2 (input, integers): range of second index to be contoured 
C 	(inclusive).
C C (input, real array): array of contour levels.
C NC (input, integer): +/- number of contour levels (less than or
C 	equal to dimension of C). If NC is positive, if it is the
C	number of contour levels, and the line-style is chosen
C	automatically as described above. If NC is negative, it is
C	minus the number of contour levels, and the current setting
C	of line-style is used for all the contours.
C TR (input, real array): dimension TR(6) defining a transformation
C	between the I,J grid of the array and the world coordinates.
C	The world coordinates of the array point A(I,J) are given by:
C	X = TR(1) + TR(2)*I + TR(3)*J
C	Y = TR(4) + TR(5)*I + TR(6)*J
C	Usually TR(3) and TR(5) are zero - unless the coordinate
C	transformation involves a rotation or shear.
C
C (7-Feb-1983)
C (24-Aug-1984) Revised to add the option of not automatically
C	setting the line-style. Sorry about the ugly way this is
C	done (negative NC); this is the least incompatible way of doing
C	it (TJP).
C-----------------------------------------------------------------------
	IMPLICIT NONE
	INTEGER  MAXEMX,MAXEMY
	PARAMETER (MAXEMX=50)
	PARAMETER (MAXEMY=50)
	INTEGER  I, I1, I2, IDIM, J, J1, J2, JDIM, NC
	INTEGER  NNX,NNY, KX,KY, IX,IY, I0,J0, II,JJ, IA,IB, JA,JB, LS
	REAL     A(IDIM,JDIM), C(*), TR(6)
	LOGICAL  STYLE
	EXTERNAL GCP
      REAL TRANS
       COMMON/TRMAT/TRANS(6)
C
C		Check arguments
C
	IF (I1.LT.1 .OR. I2.GT.IDIM .OR. I1.GE.I2 .OR.
     1     J1.LT.1 .OR. J2.GT.JDIM .OR. J1.GE.J2)
     2		CALL GRQUIT('PGCONT: invalid range I1:I2, J1:J2')
	IF (NC.EQ.0) RETURN
	STYLE = NC.GT.0
	CALL GRINQLS(LS)
C
C		Save TRANS matrix
C
	DO I=1,6
	    TRANS(I) = TR(I)
	END DO
C
C		Divide arrays into panels not exceeding
C		MAXEMX by MAXEMY for contouring by PGCONSC.
C
	NNX = I2-I1+1
	NNY = J2-J1+1
	KX = MAX(1,(NNX+MAXEMX-1)/MAXEMX)
	KY = MAX(1,(NNY+MAXEMY-1)/MAXEMY)
	I0 = MAX(NNX/(2*KX),1)
	J0 = MAX(NNY/(2*KY),1)
	IX = 2*I0
	IY = 2*J0
	II = I0+I1
	JJ = J0+J1
   20	JA = MAX(JJ-J0,J1)
	JB = MIN(JJ+J0,J2)
   30	IA = MAX(II-I0,I1)
	IB = MIN(II+I0,I2)
C
C		Draw the contours in one panel.
C
	IF (STYLE) CALL GRSETLS(1)
	DO I=1,ABS(NC)
	    IF (STYLE.AND.(C(I).LT.0.0)) CALL GRSETLS(2)
	    CALL GCONSC(A,IDIM,JDIM,IA,IB,JA,JB,C(I),GCP)
	    IF (STYLE) CALL GRSETLS(1)
	END DO
C
	II = II+IX
	IF ((II-I0.LT.I2) .AND. (II+I0.GT.1)) GOTO 30
	IX = -IX
	II = II+IX
	JJ = JJ+IY
	IF (JJ-J0.LT.J2) GOTO 20
C
	CALL GRSETLS(LS)
	RETURN
	END
